@@ -1,7 +1,9 @@
+# -*- coding: utf-8 -*-
+
 from pymysqlreplication.tests.test_basic import *
 from pymysqlreplication.tests.test_data_type import *
+from pymysqlreplication.tests.test_data_objects import *
 
 if __name__ == "__main__":
     import unittest
     unittest.main()
-
